package nl.fairbydesign.views.validation;

import com.vaadin.flow.component.Html;
import com.vaadin.flow.component.button.Button;
import com.vaadin.flow.component.dependency.CssImport;
import com.vaadin.flow.component.html.Div;
import com.vaadin.flow.component.html.Label;
import com.vaadin.flow.component.notification.Notification;
import com.vaadin.flow.component.progressbar.ProgressBar;
import com.vaadin.flow.component.textfield.TextArea;
import com.vaadin.flow.component.upload.Upload;
import com.vaadin.flow.component.upload.receivers.MemoryBuffer;
import com.vaadin.flow.router.PageTitle;
import com.vaadin.flow.router.Route;
import com.vaadin.flow.server.StreamResource;
import com.vaadin.flow.server.VaadinSession;
import nl.fairbydesign.backend.WebGeneric;
import nl.fairbydesign.backend.credentials.Credentials;
import nl.fairbydesign.backend.parsers.ExcelValidator;
import nl.fairbydesign.views.main.MainView;
import org.apache.commons.lang3.StringUtils;
import org.irods.jargon.core.connection.IRODSAccount;
import org.vaadin.olli.FileDownloadWrapper;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;

@Route(value = "validate", layout = MainView.class)
@PageTitle("Ingestion engine")
@CssImport("./styles/views/empty/empty-view.css")
public class ValidationView extends Div {

    public ValidationView() {

        Credentials credentials = (Credentials) VaadinSession.getCurrent().getAttribute("credentials");

        if (credentials != null && !credentials.isSuccess()) {
            Label label = new Label("If you can login your metadata will be automatically uploaded into the data storage system");
            add(label);
        }

        MemoryBuffer buffer = new MemoryBuffer();
        Upload upload = new Upload(buffer);
        upload.setMaxFiles(1);
        upload.setDropLabel(new Label("Upload a project excel file in .xlsx format"));

        TextArea logArea = new TextArea("Description");
        // TODO find xlsx file type
        // upload.setAcceptedFileTypes("xlsx");
        // upload.setMaxFileSize(300);

        logArea.setWidthFull();
        logArea.setPlaceholder("Log will appear here ...");
        logArea.setVisible(false);

        ProgressBar progressBar = new ProgressBar();
        progressBar.setIndeterminate(true);
        progressBar.setVisible(false);

        add(upload);
        add(logArea);

        upload.addFileRejectedListener(event -> Notification.show(event.getErrorMessage()));

        upload.addSucceededListener(succeededEvent -> {
            // When upload is successful start the excel validator!
            if (buffer.getFileName().endsWith("xlsx")) {
                logArea.setVisible(true);
                progressBar.setVisible(true);
                ExcelValidator excelValidator = new ExcelValidator();
                try {
                    IRODSAccount irodsAccount = null;
                    if (credentials != null) {
                        irodsAccount = credentials.getIrodsAccount();
                    }

                    String rdfFile = excelValidator.validate(buffer.getFileName(), buffer.getInputStream(), irodsAccount);
                    if (excelValidator.getMessage().contains("Is this an excel file?")) {
                        logArea.setValue(excelValidator.getMessage() + "\n\nInput file does not appear to be a valid XLSX file");
                    } else {
                        if (credentials != null && credentials.isSuccess()) {
                            logArea.setValue(excelValidator.getMessage() + "\n\nValidation appeared to be successful please notify the data steward of this project");
                        } else {
                            logArea.setValue(excelValidator.getMessage() + "\n\nValidation appeared to be successful.");
                            if (new File(rdfFile).exists()) {
                                // Provide download button
                                Button buttonDownload = new Button("DOWNLOAD RDF");
                                FileDownloadWrapper buttonWrapper = new FileDownloadWrapper(new StreamResource(rdfFile, () -> createResource(new File(rdfFile))));
                                buttonWrapper.wrapComponent(buttonDownload);
                                add(buttonWrapper);
                                buttonDownload.setEnabled(true);

//                                Button buttonStructure = new Button("Create folders");
//                                buttonStructure.addClickListener(click -> {
//                                    nl.munlock.backend.parsers.Structure.create(rdfFile);
//                                });
//                                add(buttonStructure);

                            }
                        }
                    }
                } catch (Exception e) {
                    logArea.setValue(
                            "\n\n=================================================================\n\n" +
                            "Your dataset did not pass the validation...\n\n" +
                            "When it is unclear please provide the following message to your data steward\n\n" +
                            e.getMessage() + "\n\n" +
                            excelValidator.getMessage() +
                            "\n\nPlease validate your mappings in the excel file\n\n" +
                            "\n\n=================================================================\n\n" +
                            StringUtils.join(e.getStackTrace(), "\n"));
                }
            }
        });

        // Set footer
        Html footerContent = WebGeneric.getTextFromResource("views/footer.html");
        Div footer = new Div();
        footer.add(footerContent);
        add(footer);
    }

    private InputStream createResource(File excelFile) {
        try {
            InputStream targetStream = new FileInputStream(excelFile);
            return targetStream;
        } catch (FileNotFoundException e) {
            e.printStackTrace();
            return null;
        }
    }
}
