package nl.fairbydesign.backend.ncbi;

import nl.fairbydesign.backend.ncbi.objects.esample.EXPERIMENTPACKAGE;
import nl.fairbydesign.backend.ncbi.objects.esample.Root;
import nl.fairbydesign.backend.ncbi.objects.esample.STUDY;
import nl.fairbydesign.backend.ncbi.objects.sheets.Study;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFFont;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.jboss.logging.Logger;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import static nl.fairbydesign.backend.parsers.ExcelGenerator.OBSERVATION_UNIT_DESCRIPTION;
import static nl.fairbydesign.backend.parsers.ExcelGenerator.OBSERVATION_UNIT_NAME;
import static nl.fairbydesign.backend.parsers.ExcelValidator.*;

public class XLSX {

    public static CellStyle headerStyle;
    public static final int defaultCellWidth = 25;
    public static CellStyle obligatoryStyle;

    public static final Logger logger = Logger.getLogger(XLSX.class);
    private static XSSFWorkbook workbook;
    private static HashSet<Root> roots;

    // List of studies
    static HashSet<Study> studies = new HashSet<>();

    /**
     * @param rootEntries sample objects from NCBI
     */
    public static void setup(HashSet<Root> rootEntries) {
        roots = rootEntries;

        // Create Excel file
        workbook = new XSSFWorkbook();
        makeHeaderCellStyle();
        makeObligatoryHeaderCellStyle();

        // Create project
        createProject();

        // Create investigation
        createInvestigation();

        // Create study
        createStudy();

        // Create observation unit
        createObservationUnit();

        // Create sample

        // Create assay

        // Write excel file
        File filename = new File("TEST" + ".xlsx");
        try {
            FileOutputStream fileOut = new FileOutputStream(filename);
            workbook.write(fileOut);
            workbook.close();
            fileOut.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private static void createObservationUnit() {
        Sheet observationUnitSheet = workbook.createSheet("ObservationUnit");
        observationUnitSheet.setDefaultColumnWidth(defaultCellWidth);
        // Create header row
        Row observationUnitSheetRow = observationUnitSheet.createRow(0);

        // Core headers
        ArrayList<String> headerList = new ArrayList<>(List.of(new String[]{OBSERVATION_UNIT_IDENTIFIER, OBSERVATION_UNIT_DESCRIPTION, OBSERVATION_UNIT_NAME, STUDY_IDENTIFIER}));

        // Other headers?

        // Write header to first row
        for (int i = 0; i < headerList.size(); i++) {
            observationUnitSheetRow.createCell(0).setCellValue(headerList.get(i));
        }

        // Build up the environment for each document
        HashSet<String> accessions = new HashSet<>();
        for (Root root : roots) {
            EXPERIMENTPACKAGE experimentPackage = root.getEXPERIMENT_PACKAGE_SET().getEXPERIMENT_PACKAGE();
            String accession = "X" + experimentPackage.getSAMPLE().getAccession();

            if (!accessions.contains(accession)) {
                observationUnitSheetRow = observationUnitSheet.createRow(accessions.size() + 1);
                // Set OBSERVATION_UNIT_IDENTIFIER
                observationUnitSheetRow.createCell(headerList.indexOf(OBSERVATION_UNIT_IDENTIFIER)).setCellValue("xxxx");
                //
                observationUnitSheetRow.createCell(headerList.indexOf(OBSERVATION_UNIT_DESCRIPTION)).setCellValue("xxxx");
                //
                observationUnitSheetRow.createCell(headerList.indexOf(OBSERVATION_UNIT_NAME)).setCellValue("xxxx");
                //
                String studyAccession = root.getEXPERIMENT_PACKAGE_SET().getEXPERIMENT_PACKAGE().getSTUDY().getAccession();
                observationUnitSheetRow.createCell(headerList.indexOf(STUDY_IDENTIFIER)).setCellValue(studyAccession);
                //
                accessions.add(accession);
            }
        }

    }

    private static void createStudy() {
        ArrayList<String> headerList = new ArrayList<>(List.of(new String[]{STUDY_IDENTIFIER, STUDY_DESCRIPTION, STUDY_TITLE, INVESTIGATION_IDENTIFIER}));

        Sheet studySheet = createSheet(headerList, "Study");

        HashSet<String> accessions = new HashSet<>();
        for (Root root : roots) {
            STUDY STUDY = root.getEXPERIMENT_PACKAGE_SET().getEXPERIMENT_PACKAGE().getSTUDY();
            if (!accessions.contains(STUDY.getAccession())) {
                logger.info("Creating study " + STUDY.getAccession());
                Row studySheetRow = studySheet.createRow(accessions.size() + 1);
                studySheetRow.createCell(0).setCellValue(STUDY.getAccession());
                studySheetRow.createCell(1).setCellValue(STUDY.getDESCRIPTOR().getSTUDY_ABSTRACT());
                studySheetRow.createCell(2).setCellValue(STUDY.getDESCRIPTOR().getSTUDY_TITLE());
                studySheetRow.createCell(3).setCellValue("INVESTIGATION_ID");
                accessions.add(STUDY.getAccession());
            }
        }
    }

    private static void createInvestigation() {

    }

    private static void createProject() {

        // Core study headers
        ArrayList<String> headerList = new ArrayList<>(List.of(new String[]{PROJECT_IDENTIFIER, PROJECT_DESCRIPTION, PROJECT_TITLE}));

        Sheet projectSheet = createSheet(headerList, "Project");

    }

    private static Sheet createSheet(ArrayList<String> headerList, String sheetName) {
        // Create study sheet
        Sheet sheet = workbook.createSheet(sheetName);
        sheet.setDefaultColumnWidth(defaultCellWidth);

        // Create header row?
        Row sheetRow = sheet.createRow(0);

        // Creating the headers
        for (int i = 0; i < headerList.size(); i++) {
            sheetRow.createCell(i).setCellValue(headerList.get(i));
        }

        // Header style
        sheetRow.cellIterator().forEachRemaining(cell -> cell.setCellStyle(headerStyle));
        return sheet;
    }

    /**
     * To create the header identical in all sheets
     */
    public static void makeHeaderCellStyle() {
        headerStyle = workbook.createCellStyle();
        headerStyle.setFillForegroundColor(IndexedColors.LIGHT_CORNFLOWER_BLUE.getIndex());
        headerStyle.setFillPattern(FillPatternType.SOLID_FOREGROUND);
        headerStyle.setAlignment(HorizontalAlignment.CENTER);
        headerStyle.setBorderBottom(BorderStyle.DOUBLE);

        XSSFFont font = workbook.createFont();
        font.setFontName("Arial");
        font.setFontHeightInPoints((short) 12);
        font.setBold(true);
        headerStyle.setFont(font);
    }

    /**
     * Header for obligatory headers
     */
    public static void makeObligatoryHeaderCellStyle() {
        obligatoryStyle = workbook.createCellStyle();
        obligatoryStyle.setFillForegroundColor(IndexedColors.LIGHT_CORNFLOWER_BLUE.getIndex());
        obligatoryStyle.setFillPattern(FillPatternType.SOLID_FOREGROUND);
        obligatoryStyle.setAlignment(HorizontalAlignment.CENTER);
        obligatoryStyle.setBorderBottom(BorderStyle.DOUBLE);

        XSSFFont font = workbook.createFont();
        font.setFontName("Arial");
        font.setFontHeightInPoints((short) 12);
        font.setBold(true);
        font.setColor(IndexedColors.RED.getIndex());
        obligatoryStyle.setFont(font);
    }
}
